/* aylet 0.3, a .AY music file player.
 * Copyright (C) 2001-2002 Russell Marks and Ian Collier.
 * See main.c for licence.
 *
 * uigp2x.c - GP2X UI code.
 */

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include <SDL.h>
#include "main.h"

#include "ui.h"
#include "gp2x.h"

char font[96][8];

static int used_ui=0;
static int retval=1;
static int display_help=0;

SDL_Surface *screen = NULL;
SDL_Surface *bg = NULL;
SDL_Joystick *joy = NULL;

static void action(enum cb_action_tag a)
{
/* only set retval if we get a zero return, i.e. we need to
 * stop current track.
 */
    if(!action_callback(a))
	retval=0;
}



static void cb_doquit(void)
{
    action(cb_quit);
}

static void ui_plot(int x, int y, int col)
{
    unsigned short *ptr = (unsigned short*) screen->pixels;
    int offs = y * (screen->pitch / sizeof(unsigned short));

    /* Colour -1 = transparent */
    if (col != -1)
    {
	ptr[offs + x] = col;
    }
}

static void ui_draw_char(int x, int y, int fc, int bc, int c)
{
    int i,j,k;

    if (c >= ' ' && c < 128)
    {
	c -= ' ';
	for (i=0; i<8; i++)
	{
	    k = font[c][i];
	    for (j=0;j<8;j++)
	    {
		if (k&(0x80>>j))
		{
		    ui_plot(x+j, y+i, fc);
		}
		else
		{
		    ui_plot(x+j, y+i, bc);
		}
	    }
	}
    }
}    

static void ui_draw_string(int x, int y, int fc, int bc, char* s)
{
    int i;
    char t[41];

    /* Truncate string to fit screen */
    strncpy(t,s,40);
    int l = strlen(s);
    if (x+(l*8) > 320)
    {
	l = (320-x)/8;
	t[l]='\0';
    }

    for (i=0; i<l; i++)
    {
	ui_draw_char(x+i*8, y, fc, bc, t[i]);
    }	
}

int keypress(SDL_Event *event)
{
    switch(event->key.keysym.sym)
    {
    case SDLK_UP:
	action(cb_prev_file);
	break;

    case SDLK_DOWN:
	action(cb_next_file);
	break;

    case SDLK_LEFT:
	action(cb_prev_track);
	break;

    case SDLK_RIGHT:
	action(cb_next_track);
	break;

    case SDLK_b:
	if (playing && !paused)
	{
	    action(cb_stop);
	}
	else
	{
	    action(cb_play);
	}
	break;

    case SDLK_a:
	action(cb_pause);
	break;

    case SDLK_l:
	action(cb_restart);
	break;

    case SDLK_q: case SDLK_ESCAPE:
	cb_doquit();
	break;

    case SDLK_r:
	action(cb_highspeed);
	break;

    case SDLK_y:
	action(cb_inc_stopafter);
	break;
    
    case SDLK_x:
	action(cb_inc_fadetime);
	break;
    
    case SDLK_SPACE:
	display_help = ~display_help;
	break;
    
    default:
	return(FALSE);      /* don't stop event if not handled */
    }

    return(TRUE);
}

int joybutton(SDL_Event *event)
{
    switch(event->jbutton.button)
    {
    case GP2X_BUTTON_UP:
	action(cb_prev_file);
	break;

    case GP2X_BUTTON_DOWN:
	action(cb_next_file);
	break;

    case GP2X_BUTTON_LEFT:
	action(cb_prev_track);
	break;

    case  GP2X_BUTTON_RIGHT:
	action(cb_next_track);
	break;

    case GP2X_BUTTON_B:
	if (playing && !paused)
	{
	    action(cb_stop);
	}
	else
	{
	    action(cb_play);
	}
	break;

    case GP2X_BUTTON_A:
	action(cb_pause);
	break;

    case GP2X_BUTTON_L:
	action(cb_restart);
	break;

    case GP2X_BUTTON_START:
	cb_doquit();
	break;

    case GP2X_BUTTON_R:
	action(cb_highspeed);
	break;

    case GP2X_BUTTON_Y:
	action(cb_inc_stopafter);
	break;
    
    case GP2X_BUTTON_X:
	action(cb_inc_fadetime);
	break;
    
    case GP2X_BUTTON_SELECT:
	display_help = ~display_help;
	break;

    default:
	return(FALSE);      /* don't stop event if not handled */
    }

    return(TRUE);
}


static void ui_initjoy(void)
{
    if (SDL_InitSubSystem (SDL_INIT_JOYSTICK) == -1)
    {
        fprintf(stderr, "Unable to initialise joystick: %s\n", SDL_GetError());
	exit(1);
    }
    
    if ( SDL_NumJoysticks() > 0)
    {
	joy = SDL_JoystickOpen(0);
    }


    if (joy)
    {
	SDL_JoystickEventState (SDL_ENABLE);	
    }
}

static void ui_initwin(void)
{
    FILE* fp;
    SDL_Surface* temp;

    // Attempt to create a 320x240 window with 16 bit pixels.
    screen = SDL_SetVideoMode(320, 240, 16, SDL_SWSURFACE);

    if ( screen == NULL ) 
    {
        fprintf(stderr, "Unable to set 320x240 video: %s\n", SDL_GetError());
        exit(1);
    }

    fp = fopen("zxspec-zx.raw", "rb");
    if (fp != NULL)
    {
	fread(font, 0x300, 1, fp);
    }

    temp = SDL_LoadBMP("gp2xbg.bmp");
    bg = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE); 
    SDL_FreeSurface(temp);

    SDL_ShowCursor(SDL_DISABLE);
}

static void ui_help()
{
    int n = 0;

    ui_draw_string(8, 104, 0xffff, -1, "Help - Select for playlist");

    ui_draw_string(8, 120+n++*10, 0, -1, "Select     - Toggle playlist/help");
    ui_draw_string(8, 120+n++*10, 0, -1, "Up/Down    - Previous/Next file");
    ui_draw_string(8, 120+n++*10, 0, -1, "Left/Right - Previous/Next track");
    ui_draw_string(8, 120+n++*10, 0, -1, "L          - Restart track");
    ui_draw_string(8, 120+n++*10, 0, -1, "R          - Toggle high speed");
    ui_draw_string(8, 120+n++*10, 0, -1, "A          - Pause");
    ui_draw_string(8, 120+n++*10, 0, -1, "B          - Start/Stop");
    ui_draw_string(8, 120+n++*10, 0, -1, "X          - Cycle stop time");
    ui_draw_string(8, 120+n++*10, 0, -1, "Y          - Cycle fade time");
    ui_draw_string(8, 120+n++*10, 0, -1, "Start      - Quit");
}

static void ui_playlist()
{
    int c;
    int n=0;
    int i=ay_file-4;
    char f[39];

    ui_draw_string(8, 104, 0xffff, -1, "Playlist - Select for help");

    if (ay_file < 4)
    {
	n = 4 - ay_file;
	i = 0;
    }

    for (; i<=ay_file+4 && i<ay_num_files; i++)
    {
	if (i == ay_file)
	{
	    c = 0x07FF;
	}
	else
	{
	    c = 0xffff;
	}

	strncpy(f, basename(ay_filenames[i]), 38);
	f[38] = '\0';
	while(strlen(f) < 38)
	{
	    f[strlen(f)] = ' ';
	}
	ui_draw_string(8,130 + n++*10, 0, c, f);
    }
}


static void ui_draw_status(char *filename,char *misc,
                           char *author,int track,char *playingstr)
{
    char t[31];
    char s[31];

    sprintf(t,"%d of %d", track, aydata.num_tracks);

    if (paused)
    {
	strcpy(s, "Paused ");
    }
    else if (playing)
    {
	strcpy(s, "Playing ");
    }
    else
    {
	strcpy(s, "Stopped ");
    }

    if (highspeed)
    {
	strcat(s, "High-speed");
    }

    ui_draw_string(32,20,0, -1, "File:");
    ui_draw_string(72,20,0, -1, basename(filename));
    ui_draw_string(32,30,0, -1, "Misc:");
    ui_draw_string(72,30,0, -1, misc);
    ui_draw_string(16,40,0, -1, "Author:");
    ui_draw_string(72,40,0, -1, author);
    ui_draw_string(24,50,0, -1, "Track:");
    ui_draw_string(72,50,0, -1, t);
    ui_draw_string(8,60,0, -1, "Playing:");
    ui_draw_string(72,60,0, -1, playingstr);
    ui_draw_string(24,70,0, -1, "State:");
    ui_draw_string(72,70,0, -1, s);
    ui_draw_string(32,80,0, -1, "Time:");

    sprintf(t, "Stop:%02d:%02d  Fade:%02d", 
	    stopafter/60, stopafter%60, fadetime);
    ui_draw_string(130, 80, 0, -1, t);

    if (display_help)
    {
	ui_help();
    }
    else
    {
	ui_playlist();
    }
}


static void ui_draw_status_timeonly(void)
{
    char t[31];
    sprintf(t, "%02d:%02d", tunetime.min, tunetime.sec);
    ui_draw_string(72,80,0, -1, t);
}


/* called per 1/50th, this deals with the usual events.
 * returns zero if we want to stop the current track.
 */
int ui_frame(void)
{
    SDL_Event event;

    retval=1;

    // Lock surface if needed
    if (SDL_LockSurface(screen) < 0) 
    {
        return 1;
    }

    /* Clear to background */
    memcpy(screen->pixels, bg->pixels, 320*240*sizeof(unsigned short));
  
    ui_draw_string(112, 4, 0xffff, -1, "AYLet.gp2x");
    
    /* update info */
    ui_draw_status(ay_filenames[ay_file],aydata.miscstr,aydata.authorstr,
		   ay_track+1,aydata.tracks[ay_track].namestr);
    
    /* update time display */
    ui_draw_status_timeonly();

    // Unlock if needed
    if (SDL_MUSTLOCK(screen)) 
        SDL_UnlockSurface(screen);

    // Tell SDL to update the whole screen
    SDL_UpdateRect(screen, 0, 0, 320, 240);    

    /* this should be the only place retval gets modified by actions */
    while (SDL_PollEvent(&event)) 
    {
	switch (event.type) 
	{
	case SDL_KEYDOWN:
	    break;
	case SDL_KEYUP:
	    keypress(&event);
	    break;
	case SDL_QUIT:
	    return(0);
	case SDL_JOYBUTTONDOWN:
	    break;
	case SDL_JOYBUTTONUP:
	    joybutton(&event);
	    break;

	}
    }    

    return(retval);
}


/* called if playback status has changed without us knowing. */
void ui_change_notify(void)
{
}


void ui_init(int argc,char **argv)
{
    used_ui=1;

    if(!use_ui)
    {
	fprintf(stderr,"gp2xaylet: warning: "
		"non-UI mode not supported by SDL version, use `aylet'.\n");
	use_ui=1;
    }

    // Initialize SDL's subsystems - in this case, only video.
    if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) 
    {
	fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
	exit(1);
    }

    // Register SDL_Quit to be called at exit; makes sure things are
    // cleaned up when we quit.
    atexit(SDL_Quit);


    ui_initjoy();
    ui_initwin();
}


void ui_end(void)
{
    if(!used_ui) return;

/* XXX can't call gtk_exit(), ui_end() is meant to return!
 * Oh well, it seems to be coping without... :-/
 */
}
